[FFmpeg-cvslog] avutil/dict: Add av_dict_iterate

Marvin Scholz git at videolan.org
Sun Nov 6 09:31:56 EET 2022


ffmpeg | branch: master | Marvin Scholz <epirat07 at gmail.com> | Sat Sep 24 16:36:55 2022 +0200| [9dad2379283cbf5842cff14c0e34b97958698201] | committer: Anton Khirnov

avutil/dict: Add av_dict_iterate

This is a more explicit iteration API rather than using the "magic"
av_dict_get(d, "", t, AV_DICT_IGNORE_SUFFIX) which is not really
trivial to grasp what it does when casually reading through code.

Signed-off-by: Anton Khirnov <anton at khirnov.net>

> http://git.videolan.org/gitweb.cgi/ffmpeg.git/?a=commit;h=9dad2379283cbf5842cff14c0e34b97958698201
---

 doc/APIchanges      |  3 +++
 libavutil/dict.c    | 19 +++++++++++++++++++
 libavutil/dict.h    | 44 ++++++++++++++++++++++++++++++++++----------
 libavutil/version.h |  2 +-
 4 files changed, 57 insertions(+), 11 deletions(-)

diff --git a/doc/APIchanges b/doc/APIchanges
index 1c4fd7efd4..038ca865ec 100644
--- a/doc/APIchanges
+++ b/doc/APIchanges
@@ -14,6 +14,9 @@ libavutil:     2021-04-27
 
 API changes, most recent first:
 
+2022-xx-xx - xxxxxxxxxx - lavu 57.42.100 - dict.h
+  Add av_dict_iterate().
+
 2022-11-03 - xxxxxxxxxx - lavu 57.41.100 - channel_layout.h
   Add AV_CH_LAYOUT_7POINT1_TOP_BACK and AV_CHANNEL_LAYOUT_7POINT1_TOP_BACK.
 
diff --git a/libavutil/dict.c b/libavutil/dict.c
index 14ad780a79..ee059d712c 100644
--- a/libavutil/dict.c
+++ b/libavutil/dict.c
@@ -20,6 +20,7 @@
 
 #include <string.h>
 
+#include "avassert.h"
 #include "avstring.h"
 #include "dict.h"
 #include "dict_internal.h"
@@ -38,6 +39,24 @@ int av_dict_count(const AVDictionary *m)
     return m ? m->count : 0;
 }
 
+const AVDictionaryEntry *av_dict_iterate(const AVDictionary *m,
+                                         const AVDictionaryEntry *prev)
+{
+    int i = 0;
+
+    if (!m)
+        return NULL;
+
+    if (prev)
+        i = prev - m->elems + 1;
+
+    av_assert2(i >= 0);
+    if (i >= m->count)
+        return NULL;
+
+    return &m->elems[i];
+}
+
 AVDictionaryEntry *av_dict_get(const AVDictionary *m, const char *key,
                                const AVDictionaryEntry *prev, int flags)
 {
diff --git a/libavutil/version.h b/libavutil/version.h
index b913f9d6c5..9b9eea2946 100644
--- a/libavutil/version.h
+++ b/libavutil/version.h
@@ -79,7 +79,7 @@
  */
 
 #define LIBAVUTIL_VERSION_MAJOR  57
-#define LIBAVUTIL_VERSION_MINOR  41
+#define LIBAVUTIL_VERSION_MINOR  42
 #define LIBAVUTIL_VERSION_MICRO 100
 
 #define LIBAVUTIL_VERSION_INT   AV_VERSION_INT(LIBAVUTIL_VERSION_MAJOR, \



More information about the ffmpeg-cvslog mailing list
diff --git a/libavutil/dict.h b/libavutil/dict.h
index 0d1afc6c64..9414cf61c0 100644
--- a/libavutil/dict.h
+++ b/libavutil/dict.h
@@ -43,9 +43,9 @@
  * an AVDictionary, simply pass an address of a NULL pointer to
  * av_dict_set(). NULL can be used as an empty dictionary wherever
  * a pointer to an AVDictionary is required.
  * Use av_dict_get() to retrieve an entry or iterate over all
  * entries and finally av_dict_free() to free the dictionary
  * and all its contents.
  *
  @code
    AVDictionary *d = NULL;           // "create" an empty dictionary
@@ -57,8 +57,8 @@
    char *v = av_strdup("value");     // you can avoid copying them like this
    av_dict_set(&d, k, v, AV_DICT_DONT_STRDUP_KEY | AV_DICT_DONT_STRDUP_VAL);
 
-   while (t = av_dict_get(d, "", t, AV_DICT_IGNORE_SUFFIX)) {
-       <....>                             // iterate over all entries in d
+   while (t = av_dict_iterate(d, t)) {
+       <....>                        // iterate over all entries in d
    }
    av_dict_free(&d);
  @endcode
@@ -89,9 +89,6 @@ typedef struct AVDictionary AVDictionary;
  * The returned entry key or value must not be changed, or it will
  * cause undefined behavior.
  *
- * To iterate through all the dictionary entries, you can set the matching key
- * to the null string "" and set the AV_DICT_IGNORE_SUFFIX flag.
- *
  * @param prev Set to the previous matching element to find the next.
  *             If set to NULL the first matching element is returned.
  * @param key matching key
@@ -101,6 +98,33 @@ typedef struct AVDictionary AVDictionary;
 AVDictionaryEntry *av_dict_get(const AVDictionary *m, const char *key,
                                const AVDictionaryEntry *prev, int flags);
 
+/**
+ * Iterate over a dictionary
+ *
+ * Iterates through all entries in the dictionary.
+ *
+ * @warning The returned AVDictionaryEntry key/value must not be changed.
+ *
+ * @warning As av_dict_set() invalidates all previous entries returned
+ * by this function, it must not be called while iterating over the dict.
+ *
+ * Typical usage:
+ * @code
+ * const AVDictionaryEntry *e = NULL;
+ * while ((e = av_dict_iterate(m, e))) {
+ *     // ...
+ * }
+ * @endcode
+ *
+ * @param m     The dictionary to iterate over
+ * @param prev  Pointer to the previous AVDictionaryEntry, NULL initially
+ *
+ * @retval AVDictionaryEntry* The next element in the dictionary
+ * @retval NULL               No more elements in the dictionary
+ */
+const AVDictionaryEntry *av_dict_iterate(const AVDictionary *m,
+                                         const AVDictionaryEntry *prev);
+
 /**
  * Get number of entries in dictionary.
  *
@@ -115,8 +139,8 @@ int av_dict_count(const AVDictionary *m);
  * Note: If AV_DICT_DONT_STRDUP_KEY or AV_DICT_DONT_STRDUP_VAL is set,
  * these arguments will be freed on error.
  *
  * Warning: Adding a new entry to a dictionary invalidates all existing entries
  * previously returned with av_dict_get.
  *
  * @param pm pointer to a pointer to a dictionary struct. If *pm is NULL
  * a dictionary struct is allocated and put in *pm.
@@ -43,9 +43,9 @@
  * av_dict_set(). NULL can be used as an empty dictionary wherever
  * a pointer to an AVDictionary is required.
- * Use av_dict_get() to retrieve an entry or iterate over all
- * entries and finally av_dict_free() to free the dictionary
- * and all its contents.
+ * Use av_dict_get() to retrieve an entry and av_dict_iterate() to
+ * iterate over all entries and finally av_dict_free() to free the
+ * dictionary and all its contents.
  *
  @code
@@ -115,8 +139,8 @@ int av_dict_count(const AVDictionary *m);
  * these arguments will be freed on error.
  *
- * Warning: Adding a new entry to a dictionary invalidates all existing entries
- * previously returned with av_dict_get.
+ * @warning Adding a new entry to a dictionary invalidates all existing entries
+ * previously returned with av_dict_get() or av_dict_iterate().
  *
  * @param pm pointer to a pointer to a dictionary struct. If *pm is NULL